@@ -40,7 +40,7 @@ void diff_restore_line(struct view *view, struct diff_state *state);
 enum status_code diff_init_highlight(struct view *view, struct diff_state *state);
 bool diff_done_highlight(struct diff_state *state);
 
-unsigned int diff_get_lineno(struct view *view, struct line *line);
+unsigned int diff_get_lineno(struct view *view, struct line *line, bool old);
 const char *diff_get_pathname(struct view *view, struct line *line);
 
 extern struct view diff_view;
