@@ -21,6 +21,8 @@
 #include "tig/display.h"
 #include "tig/watch.h"
 
+#define MAX_KEYS 2000
+
 static void set_terminal_modes(void);
 
 struct view *display[2];
@@ -187,11 +189,24 @@ static void
 redraw_display_separator(bool clear)
 {
 	if (display_sep) {
-		chtype separator = opt_line_graphics ? ACS_VLINE : '|';
+		int lineno = 0;
 
 		if (clear)
 			wclear(display_sep);
-		wbkgd(display_sep, separator + get_line_attr(NULL, LINE_TITLE_BLUR));
+		wbkgdset(display_sep, get_line_attr(NULL, LINE_TITLE_BLUR));
+
+		switch (opt_line_graphics) {
+		case GRAPHIC_ASCII:
+			while (mvwaddch(display_sep, lineno++, 0, '|') == OK);
+			break;
+		case GRAPHIC_DEFAULT:
+			while (mvwaddch(display_sep, lineno++, 0, ACS_VLINE) == OK);
+			break;
+		case GRAPHIC_UTF_8:
+			while (mvwaddstr(display_sep, lineno++, 0, "│") == OK);
+			break;
+		}
+
 		wnoutrefresh(display_sep);
 	}
 }
@@ -395,11 +410,13 @@ save_view(struct view *view, const char *path)
 		return false;
 
 	fprintf(file, "View: %s\n", view->name);
-	if (view->parent && view->parent != view)
+	if (view->prev && view->prev != view)
+		fprintf(file, "Prev: %s\n", view->prev->name);
+	if (view->parent)
 		fprintf(file, "Parent: %s\n", view->parent->name);
 	fprintf(file, "Ref: %s\n", view->ref);
 	fprintf(file, "Dimensions: height=%d width=%d\n", view->height, view->width);
-	fprintf(file, "Position: offset=%ld column=%ld lineno=%ld\n",
+	fprintf(file, "Position: offset=%lu column=%lu lineno=%lu\n",
 		view->pos.offset,
 		view->pos.col,
 		view->pos.lineno);
@@ -407,17 +424,14 @@ save_view(struct view *view, const char *path)
 	for (i = 0; i < view->lines; i++) {
 		struct line *line = &view->line[i];
 
-		fprintf(file, "line[%3zu] type=%s selected=%d\n",
+		fprintf(file, "line[%3zu] type=%s selected=%u\n",
 			i,
 			enum_name(get_line_type_name(line->type)),
 			line->selected);
 
-		if (!view->ops->get_column_data(view, line, &column_data)) {
-			fclose(file);
-			return true;
-		}
-
-		if (column_data.box) {
+		if (view->columns &&
+		    view->ops->get_column_data(view, line, &column_data) &&
+		    column_data.box) {
 			const struct box *box = column_data.box;
 			size_t j;
 			size_t offset;
@@ -623,6 +637,7 @@ init_display(void)
 	bool no_display = !!getenv("TIG_NO_DISPLAY");
 	const char *term;
 	int x, y;
+	int code;
 
 	if (!opt_tty.file)
 		die("Can't initialize display without tty");
@@ -660,6 +675,15 @@ init_display(void)
 	wbkgdset(status_win, get_line_attr(NULL, LINE_STATUS));
 	enable_mouse(opt_mouse);
 
+#ifdef NCURSES_VERSION
+	/* Disable extended keys so that esc-codes will be received
+	 * instead of extended key values (> KEY_MAX).
+	 * Then these keys can be mapped in .tigrc etc. */
+	for (code = KEY_MAX; code < MAX_KEYS; code++) {
+		keyok(code, false);
+	}
+#endif
+
 #if defined(NCURSES_VERSION_PATCH) && (NCURSES_VERSION_PATCH >= 20080119)
 	set_tabsize(opt_tab_size);
 #else
@@ -870,13 +894,13 @@ get_input(int prompt_position, struct key *key)
 			 * is set and the key value is updated to the proper
 			 * ASCII value.
 			 */
-			if (KEY_CTL('@') <= key_value && key_value <= KEY_CTL('y') &&
-			    key_value != KEY_RETURN && key_value != KEY_TAB) {
+			if (KEY_CTL('@') <= key_value && key_value <= KEY_CTL('_') &&
+			    key_value != KEY_RETURN && key_value != KEY_TAB && key_value != KEY_ESC) {
 				key->modifiers.control = 1;
 				key_value = key_value | 0x40;
 			}
 
-			if ((key_value >= KEY_MIN && key_value < KEY_MAX) || key_value < 0x1F) {
+			if ((key_value >= KEY_MIN && key_value < KEY_MAX) || key_value <= 0x1F) {
 				key->data.value = key_value;
 				return key->data.value;
 			}
