@@ -98,7 +98,8 @@ main_add_commit(struct view *view, enum line_type type, struct commit *template,
 
 	view_column_info_update(view, line);
 
-	if (opt_start_on_head && is_head_commit(commit->id))
+	if ((opt_start_on_head && is_head_commit(commit->id)) ||
+	    (view->env->goto_id[0] && !strncmp(view->env->goto_id, commit->id, SIZEOF_REV - 1)))
 		select_view_line(view, line->lineno + 1);
 
 	return commit;
@@ -248,7 +249,9 @@ main_check_argv(struct view *view, const char *argv[])
 		if (!rev_flags.with_graph)
 			state->with_graph = false;
 		arg += rev_flags.search_offset;
-		if (*arg && !*view->env->search)
+		/* Copy the pattern to search buffer only when starting
+		 * from the main view. */
+		if (*arg && !*view->ref && !view->prev)
 			string_ncopy(view->env->search, arg, strlen(arg));
 	}
 
@@ -361,8 +364,6 @@ main_get_column_data(struct view *view, const struct line *line, struct view_col
 	column_data->author = commit->author;
 	column_data->date = &commit->time;
 	column_data->id = commit->id;
-	if (state->reflogs)
-		column_data->reflog = state->reflog[line->lineno - 1];
 
 	column_data->commit_title = commit->title;
 	if (state->with_graph) {
@@ -536,22 +537,23 @@ main_read(struct view *view, struct buffer *buf, bool force_stop)
 enum request
 main_request(struct view *view, enum request request, struct line *line)
 {
-	enum open_flags flags = (view_is_displayed(view) && request != REQ_VIEW_DIFF)
+	enum open_flags flags = (request != REQ_VIEW_DIFF &&
+				 (view_is_displayed(view) ||
+				  (line->type == LINE_MAIN_COMMIT && !view_is_displayed(&diff_view)) ||
+				  line->type == LINE_STAT_UNSTAGED ||
+				  line->type == LINE_STAT_STAGED ||
+				  line->type == LINE_STAT_UNTRACKED))
 				? OPEN_SPLIT : OPEN_DEFAULT;
 
 	switch (request) {
-	case REQ_NEXT:
-	case REQ_PREVIOUS:
-		if (view_is_displayed(view) && display[0] != view)
-			return request;
-		/* Do not pass navigation requests to the branch view
-		 * when the main view is maximized. (GH #38) */
-		return request == REQ_NEXT ? REQ_MOVE_DOWN : REQ_MOVE_UP;
-
 	case REQ_VIEW_DIFF:
 	case REQ_ENTER:
-		if (view_is_displayed(view) && display[0] != view)
+		if ((view_is_displayed(view) && display[0] != view) ||
+		    (!view_is_displayed(view) && flags == OPEN_SPLIT)) {
 			maximize_view(view, true);
+			if (view->parent)
+				view->parent = NULL;
+		}
 
 		if (line->type == LINE_STAT_UNSTAGED
 		    || line->type == LINE_STAT_STAGED)
@@ -592,9 +594,16 @@ main_select(struct view *view, struct line *line)
 		string_ncopy(view->ref, commit->title, strlen(commit->title));
 		status_stage_info(view->env->status, line->type, NULL);
 	} else {
+		struct main_state *state = view->private;
 		const struct ref *ref = main_get_commit_refs(line, commit);
 
-		string_copy_rev(view->ref, commit->id);
+		if (state->reflogs) {
+			assert(state->reflogs >= line->lineno);
+			string_ncopy(view->ref, state->reflog[line->lineno - 1],
+				     strlen(state->reflog[line->lineno - 1]));
+		} else {
+			string_copy_rev(view->ref, commit->id);
+		}
 		if (ref)
 			ref_update_env(view->env, ref, true);
 	}
