@@ -594,10 +594,24 @@ parse_option(struct option_info *option, const char *prefix, const char *arg)
 		return parse_step(option->value, arg);
 
 	if (!strncmp(option->type, "enum", 4)) {
-		const char *type = option->type + STRING_SIZE("enum ");
-		const struct enum_map *map = find_enum_map(type);
+		if (!strcmp(name, "line-graphics") && !strcasecmp(arg, "auto")) {
+			const char *locale;
+			int *value = option->value;
 
-		return parse_enum(name, option->value, arg, map);
+			if ((((locale = getenv("LC_ALL")) && *locale) ||
+			     ((locale = getenv("LC_CTYPE")) && *locale) ||
+			     ((locale = getenv("LANG")) && *locale)) &&
+			    (strstr(locale, "UTF") || strstr(locale, "utf")))
+				*value = GRAPHIC_UTF_8;
+			else
+				*value = GRAPHIC_DEFAULT;
+			return SUCCESS;
+		} else {
+			const char *type = option->type + STRING_SIZE("enum ");
+			const struct enum_map *map = find_enum_map(type);
+
+			return parse_enum(name, option->value, arg, map);
+		}
 	}
 
 	if (!strcmp(option->type, "int")) {
@@ -606,7 +620,7 @@ parse_option(struct option_info *option, const char *prefix, const char *arg)
 			int *value = option->value;
 
 			/* We try to parse it as a boolean (and set the
-			 * value to 0 if fale), otherwise we parse it as
+			 * value to 0 if false), otherwise we parse it as
 			 * an integer and use the given value. */
 			if (parse_bool(&enabled, arg) == SUCCESS) {
 				if (!enabled) {
@@ -617,6 +631,16 @@ parse_option(struct option_info *option, const char *prefix, const char *arg)
 			}
 		}
 
+		if (strstr(name, "-maxwidth") && strchr(arg, '%') &&
+		    parse_int(option->value, arg, 0, 100) == SUCCESS) {
+			int *value = option->value;
+
+			/* Use negative values to signify maxwidth is
+			 * not fixed but is a % of the view width. */
+			*value *= -1;
+			return SUCCESS;
+		}
+
 		if (!strcmp(name, "line-number-interval") ||
 		    !strcmp(name, "tab-size"))
 			return parse_int(option->value, arg, 1, 1024);
